@@ -89,7 +89,7 @@ gedit_cmd_file_save (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 	if (active_child == NULL)
 		return;
 	
-	gedit_file_save (active_child);
+	gedit_file_save (active_child, TRUE);
 }
 
 void 
@@ -289,12 +289,11 @@ gedit_cmd_edit_clear (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 void
 gedit_cmd_edit_select_all (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	GeditView* active_view;
+	GeditDocument* active_doc;
 
-	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
-	g_return_if_fail (active_view);
-	
-	gedit_view_select_all (active_view); 
+	active_doc = gedit_get_active_document ();
+
+	gedit_document_set_selection (active_doc, 0, -1); 
 }
 
 void 
@@ -451,8 +450,8 @@ gedit_cmd_help_about (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 		GdkPixbuf* temp_pixbuf = NULL;
 
 		temp_pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
-					 gdk_pixbuf_get_width (pixbuf) / 2,
-					 gdk_pixbuf_get_height (pixbuf) / 2,
+					 gdk_pixbuf_get_width (pixbuf),
+					 gdk_pixbuf_get_height (pixbuf),
 					 GDK_INTERP_HYPER);
 		g_object_unref (pixbuf);
 
