@@ -74,6 +74,10 @@
 
 #define GPM_TABS_DIR			GPM_PREFS_DIR "/editor/tabs"
 #define GPM_TABS_SIZE			GPM_TABS_DIR "/tabs_size"
+#define GPM_INSERT_SPACES		GPM_TABS_DIR "/insert_spaces"
+
+#define GPM_AUTO_INDENT_DIR		GPM_PREFS_DIR "/editor/auto_indent"
+#define GPM_AUTO_INDENT			GPM_AUTO_INDENT_DIR "/auto_indent"
 
 #define GPM_LINE_NUMBERS_DIR		GPM_PREFS_DIR "/editor/line_numbers"
 #define GPM_DISPLAY_LINE_NUMBERS 	GPM_LINE_NUMBERS_DIR "/display_line_numbers"
@@ -134,6 +138,9 @@
 #define GPM_DEFAULT_WRAP_MODE		"GTK_WRAP_WORD"
 
 #define GPM_DEFAULT_TABS_SIZE		8
+#define GPM_DEFAULT_INSERT_SPACES	0 /* FALSE */
+
+#define GPM_DEFAULT_AUTO_INDENT		0 /* FALSE */
 
 #define GPM_DEFAULT_DISPLAY_LINE_NUMBERS 0 /* FALSE */
 
@@ -887,6 +894,16 @@ DEFINE_INT_PREF (tabs_size,
 		 GPM_TABS_SIZE, 
 		 GPM_DEFAULT_TABS_SIZE);
 
+/* Insert spaces */
+DEFINE_BOOL_PREF (insert_spaces, 
+		  GPM_INSERT_SPACES, 
+		  GPM_DEFAULT_INSERT_SPACES)
+
+/* Auto indent */
+DEFINE_BOOL_PREF (auto_indent, 
+		  GPM_AUTO_INDENT, 
+		  GPM_DEFAULT_AUTO_INDENT)
+
 /* Display line numbers */
 DEFINE_BOOL_PREF (display_line_numbers, 
 		  GPM_DISPLAY_LINE_NUMBERS, 
@@ -1536,7 +1553,7 @@ gedit_prefs_manager_save_window_size_and_state (BonoboWindow *window)
 
 
 /* Encodings */
-GSList const *
+GSList *
 gedit_prefs_manager_get_encodings (void)
 {
 	GSList *strings;
@@ -1551,22 +1568,7 @@ gedit_prefs_manager_get_encodings (void)
 				GCONF_VALUE_STRING, 
 				NULL);
 
-	if (strings == NULL)
-	{
-		gint i;
-		gchar* def [] = GPM_DEFAULT_ENCODINGS;
-
-		for (i = 0; def [i] != NULL; i++)
-		{
-			const GeditEncoding *enc;
-
-			enc = gedit_encoding_get_from_charset (def [i]);
-			
-			if (enc != NULL)
-				res = g_slist_prepend (res, (gpointer)enc);
-		}
-	}
-	else
+	if (strings != NULL)
 	{	
 		GSList *tmp;
 		const GeditEncoding *enc;
@@ -1590,23 +1592,46 @@ gedit_prefs_manager_get_encodings (void)
 		}
 
 		g_slist_foreach (strings, (GFunc) g_free, NULL);
-		g_slist_free (strings);           
-	}
+		g_slist_free (strings);    
 
+	 	res = g_slist_reverse (res);
+	}
 	return res;
 }
 
 void
 gedit_prefs_manager_set_encodings (const GSList *encs)
 {	
-	g_return_if_fail (encs != NULL);
+	GSList *list = NULL;
 	
 	g_return_if_fail (gedit_prefs_manager != NULL);
 	g_return_if_fail (gedit_prefs_manager->gconf_client != NULL);
 	g_return_if_fail (gedit_prefs_manager_encodings_can_set ());
 
-	/* TODO */
+	while (encs != NULL)
+	{
+		const GeditEncoding *enc;
+		const gchar *charset;
+		
+		enc = (const GeditEncoding *)encs->data;
 
+		charset = gedit_encoding_get_charset (enc);
+		g_return_if_fail (charset != NULL);
+
+		list = g_slist_prepend (list, (gpointer)charset);
+
+		encs = g_slist_next (encs);
+	}
+
+	list = g_slist_reverse (list);
+		
+	gconf_client_set_list (gedit_prefs_manager->gconf_client,
+			GPM_ENCODINGS,
+			GCONF_VALUE_STRING,
+		       	list,
+			NULL);	
+	
+	g_slist_free (list);
 }
 
 gboolean
